import { loadOptions } from "@babel/core";
import { setDefaultBy, WebpackConfigurer } from "@webpackery/core";
import { omit } from "lodash";
import { TsconfigPathsPlugin } from "tsconfig-paths-webpack-plugin";
import { InjectEmptyExports } from "./inject-empty-exports";
import { findTsConfigFile, readTsConfigFile } from "./ts-utils";
const omitKeys = [
    "extensions",
    "tsconfigPath",
    "useReact",
    "useTypesScript",
];
export class BabelConfigurer extends WebpackConfigurer {

    prepare(props, context) {
        const { production } = context;

        props = Object.assign({}, props);
        props.options = Object.assign({ sourceMaps: production, envName: production ? "production" : "development" }, props.options);

        const loaded = loadOptions(Object.assign({ filename: __filename }, props.options));

        setDefaultBy(props, "useReact", isReact, loaded.plugins);
        setDefaultBy(props, "useTypesScript", isTypeScript, loaded.plugins);
        setDefaultBy(props, "extensions", resolveDefaultExtensions, props);
        setDefaultBy(props, "test", extensionsToRegex, props.extensions);
        return props;
    }

    configure(context, props) {
        const custom = {
            plugins: [],
        };

        if (props.useTypesScript) {
            const ts = require("typescript");
            const tsconfigPath = props.tsconfigPath || findTsConfigFile(ts, context.config.context);
            if (tsconfigPath) {
                const tsconfig = readTsConfigFile(ts, tsconfigPath);
                const { compilerOptions } = tsconfig;
                const { paths, preserveConstEnums } = compilerOptions;

                if (paths)
                    context.resolve.plugin(new TsconfigPathsPlugin({
                        extensions: props.extensions,
                        configFile: tsconfigPath,
                    }));

                custom.plugins.push(constEnumPlugin(preserveConstEnums), InjectEmptyExports);
            }
        }
        const rule = Object.assign(Object.assign({ loader: require.resolve("./babel-custom-loader") }, omit(props, omitKeys)), { options: Object.assign({ custom }, props.options) });
        context.module.rule(rule);
        context.resolve.extension(...props.extensions);
    }
}
function extensionsToRegex(extensions) {
    if (extensions && extensions.length) {
        const group = extensions.map(x => x.slice(1)).join("|");
        const pattern = `\.(${group})$`;
        return new RegExp(pattern);
    }
}
function resolveDefaultExtensions(props) {
    const { useReact, useTypesScript } = props;
    return [
        ".js", ".es6", ".es", ".mjs",
        useReact && ".jsx",
        useTypesScript && [".ts", ".d.ts"],
        useTypesScript && useReact && ".tsx",
    ].filter(Boolean).flat();
}
function isReact(plugins) {
    return plugins.some(isReactPlugin);
}
function isTypeScript(plugins) {
    return plugins.some(isTypeScriptPlugin);
}

function isTypeScriptPlugin(plugin) {
    return plugin.key.startsWith("transform-typescript");
}

function isReactPlugin(plugin) {
    return plugin.key.startsWith("transform-react");
}

function constEnumPlugin(preserveConstEnums) {
    const transform = preserveConstEnums ? "removeConst" : "constObject";
    return ["babel-plugin-const-enum", { transform }];
}
//# sourceMappingURL=babel-configurer.js.map
